@@ -2,11 +2,12 @@ package Catalyst::Component;
 
 use Moose;
 use Class::MOP;
-use MooseX::Adopt::Class::Accessor::Fast;
+use Class::MOP::Object;
 use Catalyst::Utils;
 use Class::C3::Adopt::NEXT;
 use MRO::Compat;
 use mro 'c3';
+use Scalar::Util qw/blessed/;
 
 with 'MooseX::Emulate::Class::Accessor::Fast';
 with 'Catalyst::ClassData';
@@ -77,8 +78,10 @@ sub COMPONENT {
     if( my $next = $self->next::can ){
       my $class = blessed $self || $self;
       my ($next_package) = Class::MOP::get_code_info($next);
-      warn "There is a COMPONENT method resolving after Catalyst::Component in ${next_package}. This behavior is deprecated and will stop working in future releases.";
-      return $next->($self, $arguments);
+      warn "There is a COMPONENT method resolving after Catalyst::Component in ${next_package}.\n";
+      warn "This behavior can no longer be supported, and so your application is probably broken.\n";
+      warn "Your linearised isa hierarchy is: " . join(', ', mro::get_linear_isa($class)) . "\n";
+      warn "Please see perldoc Catalyst::Upgrading for more information about this issue.\n";
     }
     return $self->new($c, $arguments);
 }
@@ -94,11 +97,10 @@ sub config {
     } else {
         # this is a bit of a kludge, required to make
         # __PACKAGE__->config->{foo} = 'bar';
-        # work in a subclass. If we don't have the package symbol in the
-        # current class we know we need to copy up to ours, which calling
-        # the setter will do for us.
-
-        unless ($self->meta->has_package_symbol('$_config')) {
+        # work in a subclass.
+        my $class = blessed($self) || $self;
+        my $meta = Class::MOP::get_metaclass_by_name($class);
+        unless ($meta->has_package_symbol('$_config')) {
 
             $config = $self->merge_config_hashes( $config, {} );
             $self->_config( $config );
