@@ -1,7 +1,8 @@
 package Catalyst::DispatchType::Index;
 
-use strict;
-use base qw/Catalyst::DispatchType/;
+use Moose;
+extends 'Catalyst::DispatchType';
+no Moose;
 
 =head1 NAME
 
@@ -66,4 +67,6 @@ the same terms as Perl itself.
 
 =cut
 
+__PACKAGE__->meta->make_immutable;
+
 1;
