@@ -16,6 +16,8 @@ use Scalar::Util ();
 has read_length => (is => 'rw');
 has read_position => (is => 'rw');
 
+has _prepared_write => (is => 'rw');
+
 no Moose;
 
 # Amount of data to read from input on each pass
@@ -123,11 +125,11 @@ sub finalize_error {
         $name  = "<h1>$name</h1>";
 
         # Don't show context in the dump
-        delete $c->req->{_context};
-        delete $c->res->{_context};
+        $c->req->_clear_context;
+        $c->res->_clear_context;
 
         # Don't show body parser in the dump
-        delete $c->req->{_body};
+        $c->req->_clear_body;
 
         my @infos;
         my $i = 0;
@@ -157,6 +159,8 @@ EOF
 (dk) Venligst prov igen senere
 (pl) Prosze sprobowac pozniej
 (pt) Por favor volte mais tarde
+(ru) Попробуйте еще раз позже
+(ua) Спробуйте ще раз пізніше
 </pre>
 
         $name = '';
@@ -311,10 +315,10 @@ sub prepare_body {
 
     if ( my $length = $self->read_length ) {
         my $request = $c->request;
-        unless ( $request->{_body} ) {
+        unless ( $request->_body ) {
             my $type = $request->header('Content-Type');
-            $request->{_body} = HTTP::Body->new( $type, $length );
-            $request->{_body}->tmpdir( $c->config->{uploadtmp} )
+            $request->_body(HTTP::Body->new( $type, $length ));
+            $request->_body->tmpdir( $c->config->{uploadtmp} )
               if exists $c->config->{uploadtmp};
         }
         
@@ -332,7 +336,7 @@ sub prepare_body {
     }
     else {
         # Defined but will cause all body code to be skipped
-        $c->request->{_body} = 0;
+        $c->request->_body(0);
     }
 }
 
@@ -345,7 +349,7 @@ Add a chunk to the request body.
 sub prepare_body_chunk {
     my ( $self, $c, $chunk ) = @_;
 
-    $c->request->{_body}->add($chunk);
+    $c->request->_body->add($chunk);
 }
 
 =head2 $self->prepare_body_parameters($c)
@@ -357,9 +361,9 @@ Sets up parameters from body.
 sub prepare_body_parameters {
     my ( $self, $c ) = @_;
     
-    return unless $c->request->{_body};
+    return unless $c->request->_body;
     
-    $c->request->body_parameters( $c->request->{_body}->param );
+    $c->request->body_parameters( $c->request->_body->param );
 }
 
 =head2 $self->prepare_connection($c)
@@ -509,9 +513,9 @@ sub prepare_uploads {
     my ( $self, $c ) = @_;
 
     my $request = $c->request;
-    return unless $request->{_body};
+    return unless $request->_body;
 
-    my $uploads = $request->{_body}->upload;
+    my $uploads = $request->_body->upload;
     my $parameters = $request->parameters;
     foreach my $name (keys %$uploads) {
         my $files = $uploads->{$name};
@@ -618,9 +622,9 @@ Writes the buffer to the client.
 sub write {
     my ( $self, $c, $buffer ) = @_;
 
-    unless ( $self->{_prepared_write} ) {
+    unless ( $self->_prepared_write ) {
         $self->prepare_write($c);
-        $self->{_prepared_write} = 1;
+        $self->_prepared_write(1);
     }
     
     return 0 if !defined $buffer;
