@@ -34,16 +34,19 @@ has headers => (
   lazy => 1,
 );
 
-#Moose ToDo:
-#can we lose the before modifiers which just call prepare_body ?
-#they are wasteful, slow us down and feel cluttery.
+# Moose TODO:
+# - Can we lose the before modifiers which just call prepare_body ?
+#   they are wasteful, slow us down and feel cluttery.
 # Can we call prepare_body at BUILD time?
-# Can we make _body an attribute and have the rest of these lazy build from there?
+# Can we make _body an attribute, have the rest of 
+# these lazy build from there and kill all the direct hash access
+# in Catalyst.pm and Engine.pm?
 
 has _context => (
   is => 'rw',
   weak_ref => 1,
   handles => ['read'],
+  clearer => '_clear_context',
 );
 
 has body_parameters => (
@@ -61,16 +64,9 @@ before body_parameters => sub {
 has uploads => (
   is => 'rw',
   required => 1,
-  lazy => 1,
   default => sub { {} },
 );
 
-# modifier was a noop (groditi)
-# before uploads => sub {
-#   my ($self) = @_;
-#   #$self->_context->prepare_body;
-# };
-
 has parameters => (
   is => 'rw',
   required => 1,
@@ -80,7 +76,6 @@ has parameters => (
 
 before parameters => sub {
   my ($self, $params) = @_;
-  #$self->_context->prepare_body();
   if ( $params && !ref $params ) {
     $self->_context->log->warn(
         "Attempt to retrieve '$params' with req->params(), " .
@@ -100,14 +95,17 @@ has base => (
   },
 );
 
-has body => (
-  is => 'rw'
+has _body => (
+  is => 'rw', clearer => '_clear_body', predicate => '_has_body',
 );
-
-before body => sub {
-  my ($self) = @_;
+# Eugh, ugly. Should just be able to rename accessor methods to 'body'
+#             and provide a custom reader.. 
+sub body {
+  my $self = shift;
   $self->_context->prepare_body();
-};
+  $self->_body(@_) if scalar @_;
+  return blessed $self->_body ? $self->_body->body : $self->_body;
+}
 
 has hostname => (
   is        => 'rw',
@@ -119,6 +117,8 @@ has hostname => (
   },
 );
 
+has _path => ( is => 'rw', predicate => '_has_path', clearer => '_clear_path' );
+
 no Moose;
 
 sub args            { shift->arguments(@_) }
@@ -408,17 +408,17 @@ sub path {
 
     if (@params) {
         $self->uri->path(@params);
-        undef $self->{path};
+        $self->_clear_path;
     }
-    elsif ( defined( my $path = $self->{path} ) ) {
-        return $path;
+    elsif ( $self->_has_path ) {
+        return $self->_path;
     }
     else {
         my $path     = $self->uri->path;
         my $location = $self->base->path;
         $path =~ s/^(\Q$location\E)?//;
         $path =~ s/^\///;
-        $self->{path} = $path;
+        $self->_path($path);
 
         return $path;
     }
